from django.contrib import admin
from django.utils.functional import lazy
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from les_assets_generator.app.models import Asset, Font, Parameter

mark_safe_lazy = lazy(mark_safe, str)


class ParameterTabularInline(admin.TabularInline):
    model = Parameter
    extra = 1


class AssetAdmin(admin.ModelAdmin):
    inlines = [
        ParameterTabularInline,
    ]
    readonly_fields = ("example_url",)
    list_display = (
        "title",
        "example_url",
    )

    @admin.display(description=_("example url"))
    def example_url(self, instance):
        if instance.title:
            url = instance.generate_example_url()
            r = mark_safe_lazy(f"<a href='{url}'>{url}</a>")
        else:
            message = _("Can't determine example URL for now")
            r = mark_safe_lazy(f"<span>{message}</span>")
        return r


admin.site.register(Font)
admin.site.register(Asset, AssetAdmin)



import os
from urllib import parse

from colorfield.fields import ColorField
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.functions import Length
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

models.FileField.register_lookup(Length)


class Font(models.Model):
    name = models.CharField(_("name"), primary_key=True, max_length=200)
    font = models.FileField(_("font"), upload_to="fonts", blank=True, null=True)
    font_url = models.URLField(_("font url"), blank=True, null=True)

    def __str__(self):
        return self.name

    def clean(self, *args, **kwargs):
        if (self.font and self.font_url) or (not self.font and self.font_url is None):
            raise ValidationError(_("You must fill font url or font upload"))
        return super().clean(*args, **kwargs)

    class Meta:
        constraints = [
            models.CheckConstraint(
                check=models.Q(font__length=0, font_url__isnull=False)
                | models.Q(font__length__gt=0, font_url__isnull=True),
                name=_("You must fill font url or font upload"),
            ),
        ]


class Asset(models.Model):
    title = models.CharField(_("title"), primary_key=True, max_length=200)
    picture = models.ImageField(_("picture"), upload_to="assets", blank=True, null=True)
    picture_url = models.URLField(_("picture url"), blank=True, null=True)
    authentication = models.BooleanField(_("authentication required"), default=True)

    def __str__(self):
        return self.title

    def clean(self, *args, **kwargs):
        if (self.picture and self.picture_url) or (
            not self.picture and self.picture_url is None
        ):
            raise ValidationError(_("You must fill picture url or picture upload"))
        return super().clean(*args, **kwargs)

    def generate_example_url(self):
        url = ""
        args = {}
        params = self.assets.all()
        for param in params:
            args[param.name] = param.name
        if len(params) > 0:
            url = f"{os.environ['DEFAULT_DOMAIN']}{reverse('generate', kwargs={'title': self.title})}?{parse.urlencode(args)}"
        return url

    class Meta:
        verbose_name = _("asset")
        verbose_name_plural = _("assets")

        constraints = [
            models.CheckConstraint(
                check=models.Q(picture__length=0, picture_url__isnull=False)
                | models.Q(picture__length__gt=0, picture_url__isnull=True),
                name=_("You must fill picture url or picture upload"),
            ),
        ]


class Parameter(models.Model):
    asset = models.ForeignKey(
        Asset, on_delete=models.CASCADE, related_name="assets", verbose_name=_("assets")
    )
    font = models.ForeignKey(
        Font, on_delete=models.CASCADE, related_name="assets", verbose_name=_("font")
    )
    name = models.CharField(_("name"), max_length=200)
    color = ColorField(verbose_name=_("color"))
    font_size = models.IntegerField(_("font size"), default=0)
    x = models.IntegerField(default=0)
    y = models.IntegerField(default=0)
    mandatory = models.BooleanField(_("mandatory"), default=True)

    def __str__(self):
        return self.name

    class Meta:
        unique_together = (
            "asset",
            "name",
        )
        verbose_name = _("parameter")
        verbose_name_plural = _("parameters")



import os

from allauth.socialaccount.adapter import DefaultSocialAccountAdapter


class CustomSocialAccountAdapter(DefaultSocialAccountAdapter):
    def is_open_for_signup(self, request, sociallogin):
        u = sociallogin.user
        return u.email.split("@")[1] == os.getenv("GOOGLE_DOMAIN", "")



import os

from les_assets_generator.assets_generator.settings.base import *  # noqa: F401,F403

os.environ["DJANGO_SETTINGS_MODULE"] = "assets_generator.settings.production"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.getenv("SECRET_KEY")

# SECURITY WARNING: define the correct hosts in production!
ALLOWED_HOSTS = [
    os.getenv("DJANGO_ALLOWED_HOST"),
]



from io import BytesIO
from urllib.error import HTTPError
from urllib.request import urlopen

from django.core.exceptions import PermissionDenied, ValidationError
from django.core.validators import URLValidator
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import gettext_lazy as _
from PIL import Image, ImageDraw, ImageFont

from les_assets_generator.app.models import Asset, Parameter


def index(request):
    return redirect("https://github.com/lyon-esport/assets-generator")


def generate(request, title: str):
    asset = get_object_or_404(Asset.objects.select_related(), pk=title)
    if asset.authentication and not request.user.is_authenticated:
        raise PermissionDenied()
    else:
        params = Parameter.objects.select_related().filter(asset=asset)

        img = Image.open(asset.picture)
        draw = ImageDraw.Draw(img)

        for param in params:
            param_value = request.GET.get(param.name)
            if param.mandatory and param_value is None:
                return HttpResponse(_("Missing GET parameter %s" % param), status=422)
            elif param_value is not None:
                validator = URLValidator()
                try:
                    validator(param_value)
                except ValidationError:
                    try:
                        if param.font.font_url:
                            font_file = urlopen(param.font.font_url)
                        else:
                            font_file = param.font.font.path
                        font = ImageFont.truetype(font_file, param.font_size)
                    except OSError:
                        return HttpResponse(
                            _("Font %s not supported" % param.font),
                            status=422,
                        )
                    draw.text(
                        (param.x, param.y),
                        param_value,
                        font=font,
                        anchor="ms",
                        fill=param.color,
                    )
                else:
                    try:
                        image_to_paste = Image.open(
                            BytesIO(urlopen(param_value).read())
                        )
                        img.paste(image_to_paste, (param.x, param.y))
                    except HTTPError:
                        return HttpResponse(
                            _("Error when loading logo %s" % param_value),
                            status=422,
                        )

        byte_io = BytesIO()
        img.save(byte_io, "png")
        byte_io.seek(0)

        return HttpResponse(byte_io, content_type="image/png")
